@@ -717,6 +717,24 @@ out:
 	return ret;
 }
 
+static gboolean
+_hal_volume_temp_udi (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
+                          LibHalDrive *hal_drive, LibHalVolume *hal_volume)
+{
+        const char *volume_udi;
+        gboolean ret;
+
+        ret = FALSE;
+
+        volume_udi = libhal_volume_get_udi (hal_volume);
+
+        if (strncmp (volume_udi, "/org/freedesktop/Hal/devices/temp",
+                     strlen ("/org/freedesktop/Hal/devices/temp")) == 0)
+                ret = TRUE;
+
+        return ret;
+}
+
 static gboolean
 _hal_volume_policy_show_on_desktop (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon, 
 				    LibHalDrive *hal_drive, LibHalVolume *hal_volume)
@@ -910,6 +928,9 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 		}
 	}
 
+	if ( _hal_volume_temp_udi (volume_monitor_daemon, hal_drive, hal_volume))
+		goto out;
+
 #ifdef HAL_SHOW_DEBUG
 	g_debug ("entering _hal_add_volume for\n  drive udi '%s'\n  volume udi '%s'\n",
 		 libhal_drive_get_udi (hal_drive), libhal_volume_get_udi (hal_volume));
@@ -1019,7 +1040,7 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 	}
 	
 	ret = TRUE;
-
+out:
 	return ret;
 }
 
@@ -1508,7 +1529,7 @@ _gnome_vfs_hal_mounts_modify_drive (GnomeVFSVolumeMonitorDaemon *volume_monitor_
 	GnomeVFSDrive *result;
 	LibHalContext *hal_ctx; 
 	LibHalDrive *hal_drive;
-	char path[PATH_MAX] = "/dev/";
+	char path[PATH_MAX + 5] = "/dev/";
 	char *target = path + 5;
 	int ret;
 
@@ -1562,7 +1583,7 @@ _gnome_vfs_hal_mounts_modify_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor
 	GnomeVFSVolume *result;
 	LibHalContext *hal_ctx; 
 	LibHalDrive *hal_drive;
-	char path[PATH_MAX] = "/dev/";
+	char path[PATH_MAX + 5] = "/dev/";
 	char *target = path + 5;
 	int ret;
 
