@@ -56,8 +56,9 @@ static XdgAliasList *alias_list = NULL;
 static XdgParentList *parent_list = NULL;
 static XdgDirTimeList *dir_time_list = NULL;
 static XdgCallbackList *callback_list = NULL;
-XdgMimeCache **caches = NULL;
-int n_caches = 0;
+
+XdgMimeCache **_caches = NULL;
+static int n_caches = 0;
 
 const char *xdg_mime_type_unknown = "application/octet-stream";
 
@@ -140,8 +141,9 @@ xdg_mime_init_from_directory (const char *directory)
 	  list->next = dir_time_list;
 	  dir_time_list = list;
 
-	  caches = realloc (caches, sizeof (XdgMimeCache *) * (n_caches + 1));
-	  caches[n_caches] = cache;
+	  _caches = realloc (_caches, sizeof (XdgMimeCache *) * (n_caches + 2));
+	  _caches[n_caches] = cache;
+          _caches[n_caches + 1] = NULL;
 	  n_caches++;
 
 	  return FALSE;
@@ -433,7 +435,7 @@ xdg_mime_get_mime_type_for_data (const void *data,
 
   xdg_mime_init ();
 
-  if (caches)
+  if (_caches)
     return _xdg_mime_cache_get_mime_type_for_data (data, len);
 
   mime_type = _xdg_mime_magic_lookup_data (global_magic, data, len);
@@ -445,14 +447,15 @@ xdg_mime_get_mime_type_for_data (const void *data,
 }
 
 const char *
-xdg_mime_get_mime_type_for_file (const char *file_name)
+xdg_mime_get_mime_type_for_file (const char  *file_name,
+                                 struct stat *statbuf)
 {
   const char *mime_type;
   FILE *file;
   unsigned char *data;
   int max_extent;
   int bytes_read;
-  struct stat statbuf;
+  struct stat buf;
   const char *base_name;
 
   if (file_name == NULL)
@@ -462,7 +465,7 @@ xdg_mime_get_mime_type_for_file (const char *file_name)
 
   xdg_mime_init ();
 
-  if (caches)
+  if (_caches)
     return _xdg_mime_cache_get_mime_type_for_file (file_name);
 
   base_name = _xdg_get_base_name (file_name);
@@ -471,10 +474,15 @@ xdg_mime_get_mime_type_for_file (const char *file_name)
   if (mime_type != XDG_MIME_TYPE_UNKNOWN)
     return mime_type;
 
-  if (stat (file_name, &statbuf) != 0)
-    return XDG_MIME_TYPE_UNKNOWN;
+  if (!statbuf)
+    {
+      if (stat (file_name, &buf) != 0)
+	return XDG_MIME_TYPE_UNKNOWN;
 
-  if (!S_ISREG (statbuf.st_mode))
+      statbuf = &buf;
+    }
+
+  if (!S_ISREG (statbuf->st_mode))
     return XDG_MIME_TYPE_UNKNOWN;
 
   /* FIXME: Need to make sure that max_extent isn't totally broken.  This could
@@ -485,7 +493,7 @@ xdg_mime_get_mime_type_for_file (const char *file_name)
   if (data == NULL)
     return XDG_MIME_TYPE_UNKNOWN;
         
-      file = fopen (file_name, "r");
+  file = fopen (file_name, "r");
   if (file == NULL)
     {
       free (data);
@@ -518,7 +526,7 @@ xdg_mime_get_mime_type_from_file_name (const char *file_name)
 
   xdg_mime_init ();
 
-  if (caches)
+  if (_caches)
     return _xdg_mime_cache_get_mime_type_from_file_name (file_name);
 
   mime_type = _xdg_glob_hash_lookup_file_name (global_hash, file_name);
@@ -582,7 +590,7 @@ xdg_mime_get_max_buffer_extents (void)
 {
   xdg_mime_init ();
   
-  if (caches)
+  if (_caches)
     return _xdg_mime_cache_get_max_buffer_extents ();
 
   return _xdg_mime_magic_get_buffer_extents (global_magic);
@@ -595,7 +603,7 @@ xdg_mime_unalias_mime_type (const char *mime_type)
 
   xdg_mime_init ();
 
-  if (caches)
+  if (_caches)
     return _xdg_mime_cache_unalias_mime_type (mime_type);
 
   if ((lookup = _xdg_mime_alias_list_lookup (alias_list, mime_type)) != NULL)
@@ -663,7 +671,7 @@ xdg_mime_mime_type_subclass (const char *mime,
 
   xdg_mime_init ();
 
-  if (caches)
+  if (_caches)
     return _xdg_mime_cache_mime_type_subclass (mime, base);
 
   umime = xdg_mime_unalias_mime_type (mime);
@@ -704,10 +712,14 @@ xdg_mime_list_mime_parents (const char *mime)
   char **result;
   int i, n;
 
-  if (caches)
+  if (_caches)
     return _xdg_mime_cache_list_mime_parents (mime);
 
   parents = xdg_mime_get_mime_parents (mime);
+
+  if (!parents)
+    return NULL;
+
   for (i = 0; parents[i]; i++) ;
   
   n = (i + 1) * sizeof (char *);
