@@ -123,6 +123,8 @@ typedef struct {
 	gchar *server_type; /* the response from TYPE */
 	GnomeVFSResult fivefifty; /* the result to return for an FTP 550 */
 
+	const char *list_cmd; /* the command to be used for LIST */
+
 #ifdef HAVE_GSSAPI
 	gboolean use_gssapi;
 	gss_ctx_id_t gcontext;
@@ -172,6 +174,9 @@ static GnomeVFSResult ftp_connection_acquire   (GnomeVFSURI *uri,
 static void           ftp_connection_release   (FtpConnection *conn,
 						gboolean error_release);
 
+static GnomeVFSResult get_list_command (FtpConnection   *conn,
+					GnomeVFSContext *context);
+
 static const int   control_port = 21;
 
 
@@ -245,6 +250,8 @@ ftp_response_to_vfs_result (FtpConnection *conn)
 	case 451:
 	case 551:
 	  return GNOME_VFS_ERROR_NOT_FOUND;
+	case 504:
+	  return GNOME_VFS_ERROR_BAD_PARAMETERS;
 	case 550:
 	  return conn->fivefifty;
 	case 452:
@@ -437,7 +444,7 @@ get_response (FtpConnection *conn, GnomeVFSCancellation *cancellation)
 			conf_state = (line[0] == '6' && line[1] == '3' &&  line[2] == '1');
 			
 			/* BASE64-decode the response */
-			decoded_response = radix_decode (line + 4, &decoded_response_len);
+			decoded_response = radix_decode ((guchar *)line + 4, &decoded_response_len);
 			g_free (line);
 			if (decoded_response == NULL) {
 				return GNOME_VFS_ERROR_GENERIC;
@@ -802,7 +809,8 @@ save_authn_info (GnomeVFSURI *uri,
 
 static gboolean
 query_user_for_authn_info (GnomeVFSURI *uri, 
-			   char **user, char **pass, char **keyring, gboolean *save,
+			   char **user, char **pass, char **keyring,
+			   gboolean *save, gboolean *aborted,
 			   gboolean no_username)
 {
 	GnomeVFSModuleCallbackFullAuthenticationIn in_args;
@@ -843,10 +851,10 @@ query_user_for_authn_info (GnomeVFSURI *uri,
 		*pass = g_strdup ("nobody@gnome.org");
                 goto error;
         }
-	
-        ret = !out_args.abort_auth;
-	
-        if (!ret) {
+
+	*aborted = out_args.abort_auth;
+
+	if (out_args.abort_auth) {
                 goto error;
         }
 	
@@ -875,8 +883,8 @@ query_user_for_authn_info (GnomeVFSURI *uri,
        	g_free (out_args.domain);
        	g_free (out_args.password);
        	g_free (out_args.keyring);
-	
-	return ret;
+
+	return ret && !out_args.abort_auth;
 } 
 
 static gboolean
@@ -1023,7 +1031,7 @@ ftp_kerberos_login (FtpConnection *conn,
 				return GNOME_VFS_ERROR_LOGIN_FAILED;
 			}
 			
-			decoded_token = radix_decode (conn->response_message + 5,
+			decoded_token = radix_decode ((guchar *)conn->response_message + 5,
 						      &len);
 			if (decoded_token == NULL) {
 				gss_release_name (&min_stat, &target_name);
@@ -1206,6 +1214,25 @@ try_login (GnomeVFSURI *uri,
 	return result;	
 }
 
+static void toggle_winnt_into_unix_mode(FtpConnection *conn, 
+					GnomeVFSCancellation *cancellation)
+{
+	do_basic_command (conn, "SITE DIRSTYLE", cancellation);
+
+	if (conn->response_message != NULL
+	 && strstr(conn->response_message, "is on") != NULL)
+	{
+		/*
+		 * msdos directory mode was already off, but
+		 * we just toggled it on.  toggle it back off again
+		 */
+
+		do_basic_command(conn, "SITE DIRSTYLE", cancellation);
+	}
+
+	/* server should now be in unix directory style */
+}
+
 static GnomeVFSResult 
 ftp_connection_create (FtpConnectionPool *pool,
 		       FtpConnection **connptr,
@@ -1223,6 +1250,7 @@ ftp_connection_create (FtpConnectionPool *pool,
 	gboolean got_connection;
 	gboolean ret;
 	gboolean connection_failed;
+	gboolean aborted = FALSE;
 	
 	cancellation = get_cancellation (context);
 	
@@ -1294,7 +1322,7 @@ ftp_connection_create (FtpConnectionPool *pool,
 				pool->num_connections++;
 				G_UNLOCK (connection_pools);
 				ret = query_user_for_authn_info (uri, &user, &pass, &keyring, &save_authn, 
-								 !uri_has_username); 
+								 &aborted, !uri_has_username); 
 				G_LOCK (connection_pools);
 				pool->num_connections--;
 				if (!ret) {
@@ -1304,7 +1332,12 @@ ftp_connection_create (FtpConnectionPool *pool,
 					g_free (user);
 					g_free (pass);
 					g_free (keyring);
-                                	return GNOME_VFS_ERROR_LOGIN_FAILED;
+
+					if (aborted) {
+						return GNOME_VFS_ERROR_CANCELLED;
+					} else {
+						return GNOME_VFS_ERROR_LOGIN_FAILED;
+					}
 				}
 				g_string_free (conn->response_buffer, TRUE);
                         	conn->response_buffer = g_string_new ("");
@@ -1372,7 +1405,15 @@ ftp_connection_create (FtpConnectionPool *pool,
 	if (pool->server_type == NULL) {
 		do_basic_command (conn, "SYST", cancellation);
 		pool->server_type = g_strdup (conn->response_message);
+
 	}
+
+	/* if this is a windows server, toggle it into NT mode */
+
+	if (strncmp (pool->server_type, "Windows_NT", 10) == 0) {
+		toggle_winnt_into_unix_mode(conn, cancellation);
+	}
+
 	conn->server_type = g_strdup (pool->server_type);
 
 	*connptr = conn;
@@ -1893,7 +1934,7 @@ do_seek (GnomeVFSMethod *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset *offset_return)
+	 GnomeVFSFileSize *offset_return)
 {
 	FtpConnection *conn = (FtpConnection * )method_handle;
 
@@ -1906,97 +1947,6 @@ do_tell (GnomeVFSMethod *method,
  * only one line from that string. Fill in the appropriate fields of file_info.
  * return TRUE if a directory entry was found, FALSE otherwise
  */
-static gboolean 
-winnt_ls_to_file_info (gchar *ls, GnomeVFSFileInfo *file_info, 
-		       GnomeVFSFileInfoOptions options) 
-{
-	char *mtime_str;
-	int m, d, y, h, mn;
-	const char *mime_type;
-
-	/* check parameters */
-	g_return_val_if_fail (file_info != NULL, FALSE);
-
-	/* fill in bits of valid_fields as we go along */
-	file_info->valid_fields = 0;
-
-	/* First 17 chars are DOS date */
-	file_info->mtime = 0;
-	mtime_str = g_strndup (ls, 17);
-	if (sscanf (mtime_str, "%2d-%2d-%2d  %2d:%2d",
-	                &m, &d, &y, &h, &mn) == 5) {
-		/* yes it's a dos date */
-		struct tm mtime_parts;
-		mtime_parts.tm_mon = m - 1;   /* tm_mon is zero-based */
-		mtime_parts.tm_mday = d;
-		mtime_parts.tm_year = y >= 70 ? y : y + 100;  /* handle y2k */
-		mtime_parts.tm_hour = strcasecmp (mtime_str + 15, "pm") == 0 ?
-                                        h + 12 : h;
-		mtime_parts.tm_min = mn;
-		mtime_parts.tm_sec = 0;
-		mtime_parts.tm_isdst = -1;
-		file_info->mtime = mktime (&mtime_parts);
-		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MTIME;
-	}
-	/* TODO: if there isn't a date it's probably a Unix-style ftp server
-	 * running under Windows */
-
-	g_free (mtime_str);
-
-	/* just in case client doesn't check valid_fields */
-	file_info->atime = file_info->mtime;
-	file_info->ctime = file_info->mtime;
-
-	/* filename begins in column 39 */
-	if (strlen (ls) >= 39) {
-		int i;
-		i = strcspn (ls + 39, "\r\n");
-		file_info->name = g_strndup (ls + 39, i);
-	} else {
-		file_info->name = NULL;
-		return FALSE;
-	}
-
-	/* if it's a directory, columns 24-29 contains "<DIR>" */
-	file_info->type = GNOME_VFS_FILE_TYPE_REGULAR;
-	if (strlen (ls) >= 24) {
-		char *dirflag_str;
-		dirflag_str = g_strndup (ls + 24, 5);
-		if (strcmp (dirflag_str, "<DIR>") == 0) {
-			file_info->type = GNOME_VFS_FILE_TYPE_DIRECTORY;
-		}
-		g_free (dirflag_str);
-	}
-	file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_TYPE;
-
-	/* if not a directory, we should find right-aligned size ending
-	 * at column 37 */
-	if (file_info->type == GNOME_VFS_FILE_TYPE_REGULAR &&
-	                strlen (ls) > 17) {
-		file_info->size = strtol (ls + 17, NULL, 0);
-		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SIZE;
-	}
-
-	/* mime type */
-	if (file_info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
-		mime_type = gnome_vfs_mime_type_from_name_or_default (
-		                 file_info->name,
-		                 GNOME_VFS_MIME_TYPE_UNKNOWN);
-	} else {
-		mime_type = gnome_vfs_mime_type_from_mode (S_IFDIR);
-	}
-	file_info->mime_type = g_strdup (mime_type);
-	file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
-
-	/* fill in other structures with meaningful data, even though
-	 * it may not be valid */
-	file_info->permissions = GNOME_VFS_PERM_USER_ALL |
-	                         GNOME_VFS_PERM_GROUP_ALL |
-	                         GNOME_VFS_PERM_OTHER_ALL;
-	file_info->flags = GNOME_VFS_FILE_FLAGS_NONE;
-
-	return TRUE;
-}
 
 /**
  * return TRUE if entry found, FALSE otherwise
@@ -2150,7 +2100,7 @@ unix_ls_to_file_info (gchar *ls, GnomeVFSFileInfo *file_info,
 		   would give the stat_to_file_info function this
 		   information.  Also, there may be more fields here that are not 
 		   valid that we haven't dealt with.  */
-		file_info->valid_fields |= ~(GNOME_VFS_FILE_INFO_FIELDS_DEVICE
+		file_info->valid_fields &= ~(GNOME_VFS_FILE_INFO_FIELDS_DEVICE
 					     | GNOME_VFS_FILE_INFO_FIELDS_INODE
 					     | GNOME_VFS_FILE_INFO_FIELDS_IO_BLOCK_SIZE);
 
@@ -2347,6 +2297,50 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 				 file_info, options, context);
 }
 
+/* Tries out all suitable LIST parameter combinations until one succeeds.
+ * If there is an unexpected error, it is returned and has to be handled
+ * by the caller. Note that a successful LIST command also means that the
+ * list is actually requested, which also has to be handled by the caller.
+ * */
+static GnomeVFSResult
+get_list_command (FtpConnection   *conn,
+		  GnomeVFSContext *context)
+{
+	GnomeVFSResult result;
+
+	static const char * const osx_candidates[] = { "LIST -a", NULL };
+	static const char * const non_osx_candidates[] = { "LIST -aL", "LIST -a", "LIST -L", NULL };
+
+	const char * const *candidates;
+
+	if (strstr(conn->server_type, "MACOS")) {
+		candidates = osx_candidates;
+	} else {
+		candidates = non_osx_candidates;
+	}
+
+	do {
+		result = do_transfer_command (conn, (char *) *candidates, context);
+	}
+	while (*++candidates != NULL &&
+	       result == GNOME_VFS_ERROR_BAD_PARAMETERS);
+
+	if (result == GNOME_VFS_OK) {
+		/* we found the best suitable candidate for this server. */
+		g_assert (candidates != NULL);
+
+		conn->list_cmd = *candidates;
+	} else {
+		/* all commands failed, or an unexpected error occured,
+		 * just use "LIST". */
+
+		result = do_transfer_command (conn, "LIST", context);
+		conn->list_cmd = "LIST";
+	}
+
+	return result;
+}
+
 static GnomeVFSResult
 do_open_directory (GnomeVFSMethod *method,
 		   GnomeVFSMethodHandle **method_handle,
@@ -2407,14 +2401,14 @@ do_open_directory (GnomeVFSMethod *method,
 	result = do_path_command (conn, "CWD", uri, cancellation);
 	if (result != GNOME_VFS_OK) {
 		ftp_connection_release (conn, TRUE);
+		g_string_free (dirlist, TRUE);
 		return result;
 	}
 
-	if(strstr(conn->server_type,"MACOS")) {
-		/* don't ask for symlinks from MacOS servers */
-		result = do_transfer_command (conn, "LIST", context);
+	if (conn->list_cmd != NULL) {
+		result = do_transfer_command (conn, (char *) conn->list_cmd, context);
 	} else {
-		result = do_transfer_command (conn, "LIST -L", context);
+		result = get_list_command (conn, context);
 	}
 
 	if (result != GNOME_VFS_OK) {
@@ -2509,11 +2503,7 @@ do_read_directory (GnomeVFSMethod *method,
 	while (TRUE) {
 		gboolean success;
                 
-		if (strncmp (handle->server_type, "Windows_NT", 10) == 0) {
-			success = winnt_ls_to_file_info (handle->dirlistptr, file_info,
-			                                 handle->file_info_options);
-		}
-		else if (strncmp (handle->server_type, "NETWARE", 7) == 0) {
+		if (strncmp (handle->server_type, "NETWARE", 7) == 0) {
 			success = netware_ls_to_file_info (handle->dirlistptr, file_info,
 			                                   handle->file_info_options);
 		}
